@@ -8,21 +8,24 @@ import {
   kTextureDimensions,
 } from '../../../capability_info.js';
 import {
-  kTextureFormatInfo,
-  kRegularTextureFormats,
+  ColorTextureFormat,
+  DepthStencilFormat,
+  depthStencilFormatAspectSize,
+  getBaseFormatForTextureFormat,
+  getBlockInfoForColorTextureFormat,
+  isCompressedTextureFormat,
+  isDepthTextureFormat,
+  isRegularTextureFormat,
+  isStencilTextureFormat,
   kCompressedTextureFormats,
   kDepthStencilFormats,
-  textureDimensionAndFormatCompatible,
-  depthStencilFormatAspectSize,
-  DepthStencilFormat,
-  ColorTextureFormat,
-  isCompressedTextureFormat,
-  viewCompatible,
+  kRegularTextureFormats,
   RegularTextureFormat,
-  isRegularTextureFormat,
+  textureFormatAndDimensionPossiblyCompatible,
+  textureFormatsAreViewCompatible,
 } from '../../../format_info.js';
-import { GPUTest, TextureTestMixin } from '../../../gpu_test.js';
-import { makeBufferWithContents } from '../../../util/buffer.js';
+import { AllFeaturesMaxLimitsGPUTest } from '../../../gpu_test.js';
+import * as ttu from '../../../texture_test_utils.js';
 import { checkElementsEqual } from '../../../util/check_contents.js';
 import { align } from '../../../util/math.js';
 import { physicalMipSize } from '../../../util/texture/base.js';
@@ -33,7 +36,7 @@ import { findFailedPixels } from '../../../util/texture/texture_ok.js';
 
 const dataGenerator = new DataArrayGenerator();
 
-class F extends TextureTestMixin(GPUTest) {
+class F extends AllFeaturesMaxLimitsGPUTest {
   GetInitialDataPerMipLevel(
     dimension: GPUTextureDimension,
     textureSize: Required<GPUExtent3DDict>,
@@ -41,12 +44,9 @@ class F extends TextureTestMixin(GPUTest) {
     mipLevel: number
   ): Uint8Array {
     const textureSizeAtLevel = physicalMipSize(textureSize, format, dimension, mipLevel);
-    const bytesPerBlock = kTextureFormatInfo[format].color.bytes;
-    const blockWidthInTexel = kTextureFormatInfo[format].blockWidth;
-    const blockHeightInTexel = kTextureFormatInfo[format].blockHeight;
+    const { bytesPerBlock, blockWidth, blockHeight } = getBlockInfoForColorTextureFormat(format);
     const blocksPerSubresource =
-      (textureSizeAtLevel.width / blockWidthInTexel) *
-      (textureSizeAtLevel.height / blockHeightInTexel);
+      (textureSizeAtLevel.width / blockWidth) * (textureSizeAtLevel.height / blockHeight);
 
     const byteSize = bytesPerBlock * blocksPerSubresource * textureSizeAtLevel.depthOrArrayLayers;
     return dataGenerator.generateView(byteSize);
@@ -82,6 +82,9 @@ class F extends TextureTestMixin(GPUTest) {
     dstCopyLevel: number
   ): void {
     this.skipIfTextureFormatNotSupported(srcFormat, dstFormat);
+    this.skipIfCopyTextureToTextureNotSupportedForFormat(srcFormat, dstFormat);
+    this.skipIfTextureFormatAndDimensionNotCompatible(srcFormat, dimension);
+    this.skipIfTextureFormatAndDimensionNotCompatible(dstFormat, dimension);
 
     // If we're in compatibility mode and it's a compressed texture
     // then we need to render the texture to test the results of the copy.
@@ -99,8 +102,7 @@ class F extends TextureTestMixin(GPUTest) {
       usage: GPUTextureUsage.COPY_SRC | GPUTextureUsage.COPY_DST,
       mipLevelCount,
     };
-    const srcTexture = this.device.createTexture(srcTextureDesc);
-    this.trackForCleanup(srcTexture);
+    const srcTexture = this.createTextureTracked(srcTextureDesc);
     const dstTextureDesc: GPUTextureDescriptor = {
       dimension,
       size: dstTextureSize,
@@ -108,8 +110,7 @@ class F extends TextureTestMixin(GPUTest) {
       usage: GPUTextureUsage.COPY_SRC | GPUTextureUsage.COPY_DST | extraTextureUsageFlags,
       mipLevelCount,
     };
-    const dstTexture = this.device.createTexture(dstTextureDesc);
-    this.trackForCleanup(dstTexture);
+    const dstTexture = this.createTextureTracked(dstTextureDesc);
 
     // Fill the whole subresource of srcTexture at srcCopyLevel with initialSrcData.
     const initialSrcData = this.GetInitialDataPerMipLevel(
@@ -124,9 +125,7 @@ class F extends TextureTestMixin(GPUTest) {
       dimension,
       srcCopyLevel
     );
-    const bytesPerBlock = kTextureFormatInfo[srcFormat].color.bytes;
-    const blockWidth = kTextureFormatInfo[srcFormat].blockWidth;
-    const blockHeight = kTextureFormatInfo[srcFormat].blockHeight;
+    const { bytesPerBlock, blockWidth, blockHeight } = getBlockInfoForColorTextureFormat(srcFormat);
     const srcBlocksPerRow = srcTextureSizeAtLevel.width / blockWidth;
     const srcBlockRowsPerImage = srcTextureSizeAtLevel.height / blockHeight;
     this.device.queue.writeTexture(
@@ -211,11 +210,11 @@ class F extends TextureTestMixin(GPUTest) {
       align(dstBlocksPerRow * bytesPerBlock, 4);
 
     if (isCompressedTextureFormat(dstTexture.format) && this.isCompatibility) {
-      assert(viewCompatible(this.isCompatibility, srcFormat, dstFormat));
+      assert(textureFormatsAreViewCompatible(this.device, srcFormat, dstFormat));
       // compare by rendering. We need the expected texture to match
       // the dstTexture so we'll create a texture where we supply
       // all of the data in JavaScript.
-      const expectedTexture = this.device.createTexture({
+      const expectedTexture = this.createTextureTracked({
         size: [dstTexture.width, dstTexture.height, dstTexture.depthOrArrayLayers],
         mipLevelCount: dstTexture.mipLevelCount,
         format: dstTexture.format,
@@ -225,7 +224,7 @@ class F extends TextureTestMixin(GPUTest) {
 
       // Execute the equivalent of `copyTextureToTexture`, copying
       // from `initialSrcData` to `expectedData`.
-      this.updateLinearTextureDataSubBox(dstFormat, appliedSize, {
+      ttu.updateLinearTextureDataSubBox(this, dstFormat, appliedSize, {
         src: {
           dataLayout: {
             bytesPerRow: srcBlocksPerRow * bytesPerBlock,
@@ -258,7 +257,8 @@ class F extends TextureTestMixin(GPUTest) {
         dstTextureSizeAtLevel
       );
 
-      this.expectTexturesToMatchByRendering(
+      ttu.expectTexturesToMatchByRendering(
+        this,
         dstTexture,
         expectedTexture,
         dstCopyLevel,
@@ -273,8 +273,7 @@ class F extends TextureTestMixin(GPUTest) {
       size: dstBufferSize,
       usage: GPUBufferUsage.COPY_SRC | GPUBufferUsage.COPY_DST,
     };
-    const dstBuffer = this.device.createBuffer(dstBufferDesc);
-    this.trackForCleanup(dstBuffer);
+    const dstBuffer = this.createBufferTracked(dstBufferDesc);
 
     {
       const encoder = this.device.createCommandEncoder();
@@ -436,11 +435,10 @@ class F extends TextureTestMixin(GPUTest) {
       }),
       kBufferSizeAlignment
     );
-    const outputBuffer = this.device.createBuffer({
+    const outputBuffer = this.createBufferTracked({
       size: outputBufferSize,
       usage: GPUBufferUsage.COPY_SRC | GPUBufferUsage.COPY_DST,
     });
-    this.trackForCleanup(outputBuffer);
     const encoder = this.device.createCommandEncoder();
     encoder.copyTextureToBuffer(
       {
@@ -545,8 +543,7 @@ class F extends TextureTestMixin(GPUTest) {
     for (let i = 1; i < totalCopyArrayLayers; ++i) {
       uniformBufferData[(kMinDynamicBufferOffsetAlignment / 4) * i] = i;
     }
-    const uniformBuffer = makeBufferWithContents(
-      this.device,
+    const uniformBuffer = this.makeBufferWithContents(
       uniformBufferData,
       GPUBufferUsage.COPY_DST | GPUBufferUsage.UNIFORM
     );
@@ -582,7 +579,7 @@ class F extends TextureTestMixin(GPUTest) {
     });
     const bindGroup = this.GetBindGroupForT2TCopyWithDepthTests(bindGroupLayout, copySize[2]);
 
-    const hasStencil = kTextureFormatInfo[sourceTexture.format].stencil;
+    const hasStencil = isStencilTextureFormat(sourceTexture.format);
     const encoder = this.device.createCommandEncoder();
     for (let srcCopyLayer = 0; srcCopyLayer < copySize[2]; ++srcCopyLayer) {
       const renderPass = encoder.beginRenderPass({
@@ -626,14 +623,12 @@ class F extends TextureTestMixin(GPUTest) {
     });
     const bindGroup = this.GetBindGroupForT2TCopyWithDepthTests(bindGroupLayout, copySize[2]);
 
-    const outputColorTexture = this.trackForCleanup(
-      this.device.createTexture({
-        format: 'rgba8unorm',
-        size: copySize,
-        usage: GPUTextureUsage.RENDER_ATTACHMENT | GPUTextureUsage.COPY_SRC,
-      })
-    );
-    const hasStencil = kTextureFormatInfo[destinationTexture.format].stencil;
+    const outputColorTexture = this.createTextureTracked({
+      format: 'rgba8unorm',
+      size: copySize,
+      usage: GPUTextureUsage.RENDER_ATTACHMENT | GPUTextureUsage.COPY_SRC,
+    });
+    const hasStencil = isStencilTextureFormat(destinationTexture.format);
     const encoder = this.device.createCommandEncoder();
     for (let dstCopyLayer = 0; dstCopyLayer < copySize[2]; ++dstCopyLayer) {
       // If the depth value is not expected, the color of outputColorTexture will remain Red after
@@ -798,8 +793,8 @@ g.test('color_textures,non_compressed,non_array')
       .combine('srcFormat', kRegularTextureFormats)
       .combine('dstFormat', kRegularTextureFormats)
       .filter(({ srcFormat, dstFormat }) => {
-        const srcBaseFormat = kTextureFormatInfo[srcFormat].baseFormat;
-        const dstBaseFormat = kTextureFormatInfo[dstFormat].baseFormat;
+        const srcBaseFormat = getBaseFormatForTextureFormat(srcFormat);
+        const dstBaseFormat = getBaseFormatForTextureFormat(dstFormat);
         return (
           srcFormat === dstFormat ||
           (srcBaseFormat !== undefined &&
@@ -810,8 +805,8 @@ g.test('color_textures,non_compressed,non_array')
       .combine('dimension', kTextureDimensions)
       .filter(
         ({ dimension, srcFormat, dstFormat }) =>
-          textureDimensionAndFormatCompatible(dimension, srcFormat) &&
-          textureDimensionAndFormatCompatible(dimension, dstFormat)
+          textureFormatAndDimensionPossiblyCompatible(dimension, srcFormat) &&
+          textureFormatAndDimensionPossiblyCompatible(dimension, dstFormat)
       )
       .beginSubcases()
       .expandWithParams(p => {
@@ -893,8 +888,8 @@ g.test('color_textures,compressed,non_array')
       .combine('srcFormat', kCompressedTextureFormats)
       .combine('dstFormat', kCompressedTextureFormats)
       .filter(({ srcFormat, dstFormat }) => {
-        const srcBaseFormat = kTextureFormatInfo[srcFormat].baseFormat;
-        const dstBaseFormat = kTextureFormatInfo[dstFormat].baseFormat;
+        const srcBaseFormat = getBaseFormatForTextureFormat(srcFormat);
+        const dstBaseFormat = getBaseFormatForTextureFormat(dstFormat);
         return (
           srcFormat === dstFormat ||
           (srcBaseFormat !== undefined &&
@@ -903,11 +898,6 @@ g.test('color_textures,compressed,non_array')
         );
       })
       .combine('dimension', kTextureDimensions)
-      .filter(
-        ({ dimension, srcFormat, dstFormat }) =>
-          textureDimensionAndFormatCompatible(dimension, srcFormat) &&
-          textureDimensionAndFormatCompatible(dimension, dstFormat)
-      )
       .beginSubcases()
       .combine('textureSizeInBlocks', [
         // The heights and widths in blocks are all power of 2
@@ -929,14 +919,6 @@ g.test('color_textures,compressed,non_array')
       .combine('srcCopyLevel', [0, 2])
       .combine('dstCopyLevel', [0, 2])
   )
-  .beforeAllSubcases(t => {
-    const { srcFormat, dstFormat } = t.params;
-    t.skipIfCopyTextureToTextureNotSupportedForFormat(srcFormat, dstFormat);
-    t.selectDeviceOrSkipTestCase([
-      kTextureFormatInfo[srcFormat].feature,
-      kTextureFormatInfo[dstFormat].feature,
-    ]);
-  })
   .fn(t => {
     const {
       dimension,
@@ -947,10 +929,13 @@ g.test('color_textures,compressed,non_array')
       srcCopyLevel,
       dstCopyLevel,
     } = t.params;
-    const srcBlockWidth = kTextureFormatInfo[srcFormat].blockWidth;
-    const srcBlockHeight = kTextureFormatInfo[srcFormat].blockHeight;
-    const dstBlockWidth = kTextureFormatInfo[dstFormat].blockWidth;
-    const dstBlockHeight = kTextureFormatInfo[dstFormat].blockHeight;
+    t.skipIfTextureFormatAndDimensionNotCompatible(srcFormat, dimension);
+    t.skipIfTextureFormatAndDimensionNotCompatible(dstFormat, dimension);
+    t.skipIfCopyTextureToTextureNotSupportedForFormat(srcFormat, dstFormat);
+    const { blockWidth: srcBlockWidth, blockHeight: srcBlockHeight } =
+      getBlockInfoForColorTextureFormat(srcFormat);
+    const { blockWidth: dstBlockWidth, blockHeight: dstBlockHeight } =
+      getBlockInfoForColorTextureFormat(dstFormat);
 
     t.DoCopyTextureToTextureTest(
       dimension,
@@ -985,8 +970,8 @@ g.test('color_textures,non_compressed,array')
       .combine('srcFormat', kRegularTextureFormats)
       .combine('dstFormat', kRegularTextureFormats)
       .filter(({ srcFormat, dstFormat }) => {
-        const srcBaseFormat = kTextureFormatInfo[srcFormat].baseFormat;
-        const dstBaseFormat = kTextureFormatInfo[dstFormat].baseFormat;
+        const srcBaseFormat = getBaseFormatForTextureFormat(srcFormat);
+        const dstBaseFormat = getBaseFormatForTextureFormat(dstFormat);
         return (
           srcFormat === dstFormat ||
           (srcBaseFormat !== undefined &&
@@ -997,8 +982,8 @@ g.test('color_textures,non_compressed,array')
       .combine('dimension', ['2d', '3d'] as const)
       .filter(
         ({ dimension, srcFormat, dstFormat }) =>
-          textureDimensionAndFormatCompatible(dimension, srcFormat) &&
-          textureDimensionAndFormatCompatible(dimension, dstFormat)
+          textureFormatAndDimensionPossiblyCompatible(dimension, srcFormat) &&
+          textureFormatAndDimensionPossiblyCompatible(dimension, dstFormat)
       )
       .beginSubcases()
       .combine('textureSize', [
@@ -1058,8 +1043,8 @@ g.test('color_textures,compressed,array')
       .combine('srcFormat', kCompressedTextureFormats)
       .combine('dstFormat', kCompressedTextureFormats)
       .filter(({ srcFormat, dstFormat }) => {
-        const srcBaseFormat = kTextureFormatInfo[srcFormat].baseFormat;
-        const dstBaseFormat = kTextureFormatInfo[dstFormat].baseFormat;
+        const srcBaseFormat = getBaseFormatForTextureFormat(srcFormat);
+        const dstBaseFormat = getBaseFormatForTextureFormat(dstFormat);
         return (
           srcFormat === dstFormat ||
           (srcBaseFormat !== undefined &&
@@ -1068,11 +1053,6 @@ g.test('color_textures,compressed,array')
         );
       })
       .combine('dimension', ['2d', '3d'] as const)
-      .filter(
-        ({ dimension, srcFormat, dstFormat }) =>
-          textureDimensionAndFormatCompatible(dimension, srcFormat) &&
-          textureDimensionAndFormatCompatible(dimension, dstFormat)
-      )
       .beginSubcases()
       .combine('textureSizeInBlocks', [
         // The heights and widths in blocks are all power of 2
@@ -1084,14 +1064,6 @@ g.test('color_textures,compressed,array')
       .combine('srcCopyLevel', [0, 2])
       .combine('dstCopyLevel', [0, 2])
   )
-  .beforeAllSubcases(t => {
-    const { srcFormat, dstFormat } = t.params;
-    t.skipIfCopyTextureToTextureNotSupportedForFormat(srcFormat, dstFormat);
-    t.selectDeviceOrSkipTestCase([
-      kTextureFormatInfo[srcFormat].feature,
-      kTextureFormatInfo[dstFormat].feature,
-    ]);
-  })
   .fn(t => {
     const {
       dimension,
@@ -1102,10 +1074,10 @@ g.test('color_textures,compressed,array')
       srcCopyLevel,
       dstCopyLevel,
     } = t.params;
-    const srcBlockWidth = kTextureFormatInfo[srcFormat].blockWidth;
-    const srcBlockHeight = kTextureFormatInfo[srcFormat].blockHeight;
-    const dstBlockWidth = kTextureFormatInfo[dstFormat].blockWidth;
-    const dstBlockHeight = kTextureFormatInfo[dstFormat].blockHeight;
+    const { blockWidth: srcBlockWidth, blockHeight: srcBlockHeight } =
+      getBlockInfoForColorTextureFormat(srcFormat);
+    const { blockWidth: dstBlockWidth, blockHeight: dstBlockHeight } =
+      getBlockInfoForColorTextureFormat(dstFormat);
 
     t.DoCopyTextureToTextureTest(
       dimension,
@@ -1268,10 +1240,6 @@ g.test('copy_depth_stencil')
         );
       })
   )
-  .beforeAllSubcases(t => {
-    const { format } = t.params;
-    t.selectDeviceForTextureFormatOrSkipTestCase(format);
-  })
   .fn(t => {
     const {
       format,
@@ -1281,37 +1249,34 @@ g.test('copy_depth_stencil')
       srcCopyBaseArrayLayer,
       dstCopyBaseArrayLayer,
     } = t.params;
+    t.skipIfTextureFormatNotSupported(format);
 
     const copySize: [number, number, number] = [
       srcTextureSize.width >> srcCopyLevel,
       srcTextureSize.height >> srcCopyLevel,
       srcTextureSize.depthOrArrayLayers - Math.max(srcCopyBaseArrayLayer, dstCopyBaseArrayLayer),
     ];
-    const sourceTexture = t.trackForCleanup(
-      t.device.createTexture({
-        format,
-        size: srcTextureSize,
-        usage:
-          GPUTextureUsage.COPY_SRC | GPUTextureUsage.COPY_DST | GPUTextureUsage.RENDER_ATTACHMENT,
-        mipLevelCount: srcCopyLevel + 1,
-      })
-    );
-    const destinationTexture = t.trackForCleanup(
-      t.device.createTexture({
-        format,
-        size: [
-          copySize[0] << dstCopyLevel,
-          copySize[1] << dstCopyLevel,
-          srcTextureSize.depthOrArrayLayers,
-        ] as const,
-        usage:
-          GPUTextureUsage.COPY_SRC | GPUTextureUsage.COPY_DST | GPUTextureUsage.RENDER_ATTACHMENT,
-        mipLevelCount: dstCopyLevel + 1,
-      })
-    );
+    const sourceTexture = t.createTextureTracked({
+      format,
+      size: srcTextureSize,
+      usage:
+        GPUTextureUsage.COPY_SRC | GPUTextureUsage.COPY_DST | GPUTextureUsage.RENDER_ATTACHMENT,
+      mipLevelCount: srcCopyLevel + 1,
+    });
+    const destinationTexture = t.createTextureTracked({
+      format,
+      size: [
+        copySize[0] << dstCopyLevel,
+        copySize[1] << dstCopyLevel,
+        srcTextureSize.depthOrArrayLayers,
+      ] as const,
+      usage:
+        GPUTextureUsage.COPY_SRC | GPUTextureUsage.COPY_DST | GPUTextureUsage.RENDER_ATTACHMENT,
+      mipLevelCount: dstCopyLevel + 1,
+    });
 
     let initialStencilData: undefined | Uint8Array = undefined;
-    if (kTextureFormatInfo[format].stencil) {
+    if (isStencilTextureFormat(format)) {
       initialStencilData = t.GetInitialStencilDataPerMipLevel(srcTextureSize, format, srcCopyLevel);
       t.InitializeStencilAspect(
         sourceTexture,
@@ -1321,7 +1286,7 @@ g.test('copy_depth_stencil')
         copySize
       );
     }
-    if (kTextureFormatInfo[format].depth) {
+    if (isDepthTextureFormat(format)) {
       t.InitializeDepthAspect(sourceTexture, format, srcCopyLevel, srcCopyBaseArrayLayer, copySize);
     }
 
@@ -1341,7 +1306,7 @@ g.test('copy_depth_stencil')
     );
     t.queue.submit([encoder.finish()]);
 
-    if (kTextureFormatInfo[format].stencil) {
+    if (isStencilTextureFormat(format)) {
       assert(initialStencilData !== undefined);
       t.VerifyStencilAspect(
         destinationTexture,
@@ -1351,7 +1316,7 @@ g.test('copy_depth_stencil')
         copySize
       );
     }
-    if (kTextureFormatInfo[format].depth) {
+    if (isDepthTextureFormat(format)) {
       t.VerifyDepthAspect(
         destinationTexture,
         format,
@@ -1376,15 +1341,13 @@ g.test('copy_multisampled_color')
     texture can only be 1.
   `
   )
-  .beforeAllSubcases(t => {
-    t.skipIf(t.isCompatibility, 'multisample textures are not copyable in compatibility mode');
-  })
   .fn(t => {
+    t.skipIf(t.isCompatibility, 'multisample textures are not copyable in compatibility mode');
     const textureSize = [32, 16, 1] as const;
     const kColorFormat = 'rgba8unorm';
     const kSampleCount = 4;
 
-    const sourceTexture = t.device.createTexture({
+    const sourceTexture = t.createTextureTracked({
       format: kColorFormat,
       size: textureSize,
       usage:
@@ -1393,8 +1356,7 @@ g.test('copy_multisampled_color')
         GPUTextureUsage.RENDER_ATTACHMENT,
       sampleCount: kSampleCount,
     });
-    t.trackForCleanup(sourceTexture);
-    const destinationTexture = t.device.createTexture({
+    const destinationTexture = t.createTextureTracked({
       format: kColorFormat,
       size: textureSize,
       usage:
@@ -1403,7 +1365,6 @@ g.test('copy_multisampled_color')
         GPUTextureUsage.RENDER_ATTACHMENT,
       sampleCount: kSampleCount,
     });
-    t.trackForCleanup(destinationTexture);
 
     // Initialize sourceTexture with a draw call.
     const renderPipelineForInit = t.device.createRenderPipeline({
@@ -1526,12 +1487,11 @@ g.test('copy_multisampled_color')
         },
       ],
     });
-    const expectedOutputTexture = t.device.createTexture({
+    const expectedOutputTexture = t.createTextureTracked({
       format: kColorFormat,
       size: textureSize,
       usage: GPUTextureUsage.COPY_SRC | GPUTextureUsage.RENDER_ATTACHMENT,
     });
-    t.trackForCleanup(expectedOutputTexture);
     const validationEncoder = t.device.createCommandEncoder();
     const renderPassForValidation = validationEncoder.beginRenderPass({
       colorAttachments: [
@@ -1567,28 +1527,30 @@ g.test('copy_multisampled_depth')
     texture can only be 1.
   `
   )
-  .beforeAllSubcases(t => {
-    t.skipIf(t.isCompatibility, 'multisample textures are not copyable in compatibility mode');
-  })
+  .params(u =>
+    u.combine('format', kDepthStencilFormats).filter(t => isDepthTextureFormat(t.format))
+  )
   .fn(t => {
+    const { format } = t.params;
+
+    t.skipIf(t.isCompatibility, 'multisample textures are not copyable in compatibility mode');
+    t.skipIfTextureFormatNotSupported(format);
+
     const textureSize = [32, 16, 1] as const;
-    const kDepthFormat = 'depth24plus';
     const kSampleCount = 4;
 
-    const sourceTexture = t.device.createTexture({
-      format: kDepthFormat,
+    const sourceTexture = t.createTextureTracked({
+      format,
       size: textureSize,
       usage: GPUTextureUsage.COPY_SRC | GPUTextureUsage.RENDER_ATTACHMENT,
       sampleCount: kSampleCount,
     });
-    t.trackForCleanup(sourceTexture);
-    const destinationTexture = t.device.createTexture({
-      format: kDepthFormat,
+    const destinationTexture = t.createTextureTracked({
+      format,
       size: textureSize,
       usage: GPUTextureUsage.COPY_DST | GPUTextureUsage.RENDER_ATTACHMENT,
       sampleCount: kSampleCount,
     });
-    t.trackForCleanup(destinationTexture);
 
     const vertexState: GPUVertexState = {
       module: t.device.createShaderModule({
@@ -1613,7 +1575,7 @@ g.test('copy_multisampled_depth')
       layout: 'auto',
       vertex: vertexState,
       depthStencil: {
-        format: kDepthFormat,
+        format,
         depthCompare: 'always',
         depthWriteEnabled: true,
       },
@@ -1630,6 +1592,10 @@ g.test('copy_multisampled_depth')
         depthClearValue: 0.0,
         depthLoadOp: 'clear',
         depthStoreOp: 'store',
+        ...(isStencilTextureFormat(format) && {
+          stencilLoadOp: 'clear',
+          stencilStoreOp: 'store',
+        }),
       },
     });
     renderPassForInit.setPipeline(renderPipelineForInit);
@@ -1668,7 +1634,7 @@ g.test('copy_multisampled_depth')
         targets: [{ format: kColorFormat }],
       },
       depthStencil: {
-        format: kDepthFormat,
+        format,
         depthCompare: 'equal',
         depthWriteEnabled: false,
       },
@@ -1676,19 +1642,17 @@ g.test('copy_multisampled_depth')
         count: kSampleCount,
       },
     });
-    const multisampledColorTexture = t.device.createTexture({
+    const multisampledColorTexture = t.createTextureTracked({
       format: kColorFormat,
       size: textureSize,
       usage: GPUTextureUsage.COPY_SRC | GPUTextureUsage.RENDER_ATTACHMENT,
       sampleCount: kSampleCount,
     });
-    t.trackForCleanup(multisampledColorTexture);
-    const colorTextureAsResolveTarget = t.device.createTexture({
+    const colorTextureAsResolveTarget = t.createTextureTracked({
       format: kColorFormat,
       size: textureSize,
       usage: GPUTextureUsage.COPY_SRC | GPUTextureUsage.RENDER_ATTACHMENT,
     });
-    t.trackForCleanup(colorTextureAsResolveTarget);
 
     const encoderForVerify = t.device.createCommandEncoder();
     const renderPassForVerify = encoderForVerify.beginRenderPass({
@@ -1705,6 +1669,10 @@ g.test('copy_multisampled_depth')
         view: destinationTexture.createView(),
         depthLoadOp: 'load',
         depthStoreOp: 'store',
+        ...(isStencilTextureFormat(format) && {
+          stencilLoadOp: 'clear',
+          stencilStoreOp: 'store',
+        }),
       },
     });
     renderPassForVerify.setPipeline(renderPipelineForVerify);
@@ -1717,3 +1685,11 @@ g.test('copy_multisampled_depth')
       exp: { R: 0.0, G: 1.0, B: 0.0, A: 1.0 },
     });
   });
+
+g.test('copy_multisampled_stencil')
+  .desc(
+    `
+  Validate the correctness of copyTextureToTexture() with multisampled stencil formats.
+    `
+  )
+  .unimplemented();
