@@ -4,6 +4,7 @@ createComputePipeline and createComputePipelineAsync validation tests.
 Note: entry point matching tests are in shader_module/entry_point.spec.ts
 `;
 
+import { AllFeaturesMaxLimitsGPUTest } from '../.././gpu_test.js';
 import { makeTestGroup } from '../../../common/framework/test_group.js';
 import { keysOf } from '../../../common/util/data_tables.js';
 import { kValue } from '../../util/constants.js';
@@ -15,9 +16,9 @@ import {
   getAPIBindGroupLayoutForResource,
   doResourcesMatch,
 } from './utils.js';
-import { ValidationTest } from './validation_test.js';
+import * as vtu from './validation_test_utils.js';
 
-class F extends ValidationTest {
+class F extends AllFeaturesMaxLimitsGPUTest {
   getShaderModule(
     shaderStage: TShaderStage = 'compute',
     entryPoint: string = 'main'
@@ -40,7 +41,7 @@ Call the API with valid compute shader and matching valid entryPoint, making sur
   .params(u => u.combine('isAsync', [true, false]))
   .fn(t => {
     const { isAsync } = t.params;
-    t.doCreateComputePipelineTest(isAsync, true, {
+    vtu.doCreateComputePipelineTest(t, isAsync, true, {
       layout: 'auto',
       compute: { module: t.getShaderModule('compute', 'main'), entryPoint: 'main' },
     });
@@ -55,10 +56,10 @@ Tests calling createComputePipeline(Async) with a invalid compute shader, and ch
   .params(u => u.combine('isAsync', [true, false]))
   .fn(t => {
     const { isAsync } = t.params;
-    t.doCreateComputePipelineTest(isAsync, false, {
+    vtu.doCreateComputePipelineTest(t, isAsync, false, {
       layout: 'auto',
       compute: {
-        module: t.createInvalidShaderModule(),
+        module: vtu.createInvalidShaderModule(t),
         entryPoint: 'main',
       },
     });
@@ -85,7 +86,7 @@ and check that the APIs only accept compute shader.
         entryPoint: 'main',
       },
     };
-    t.doCreateComputePipelineTest(isAsync, shaderModuleStage === 'compute', descriptor);
+    vtu.doCreateComputePipelineTest(t, isAsync, shaderModuleStage === 'compute', descriptor);
   });
 
 g.test('shader_module,device_mismatch')
@@ -93,9 +94,7 @@ g.test('shader_module,device_mismatch')
     'Tests createComputePipeline(Async) cannot be called with a shader module created from another device'
   )
   .paramsSubcasesOnly(u => u.combine('isAsync', [true, false]).combine('mismatched', [true, false]))
-  .beforeAllSubcases(t => {
-    t.selectMismatchedDeviceOrSkipTestCase(undefined);
-  })
+  .beforeAllSubcases(t => t.usesMismatchedDevice())
   .fn(t => {
     const { isAsync, mismatched } = t.params;
 
@@ -113,7 +112,7 @@ g.test('shader_module,device_mismatch')
       },
     };
 
-    t.doCreateComputePipelineTest(isAsync, !mismatched, descriptor);
+    vtu.doCreateComputePipelineTest(t, isAsync, !mismatched, descriptor);
   });
 
 g.test('pipeline_layout,device_mismatch')
@@ -121,9 +120,7 @@ g.test('pipeline_layout,device_mismatch')
     'Tests createComputePipeline(Async) cannot be called with a pipeline layout created from another device'
   )
   .paramsSubcasesOnly(u => u.combine('isAsync', [true, false]).combine('mismatched', [true, false]))
-  .beforeAllSubcases(t => {
-    t.selectMismatchedDeviceOrSkipTestCase(undefined);
-  })
+  .beforeAllSubcases(t => t.usesMismatchedDevice())
   .fn(t => {
     const { isAsync, mismatched } = t.params;
     const sourceDevice = mismatched ? t.mismatchedDevice : t.device;
@@ -138,7 +135,7 @@ g.test('pipeline_layout,device_mismatch')
       },
     };
 
-    t.doCreateComputePipelineTest(isAsync, !mismatched, descriptor);
+    vtu.doCreateComputePipelineTest(t, isAsync, !mismatched, descriptor);
   });
 
 g.test('limits,workgroup_storage_size')
@@ -176,7 +173,7 @@ Tests calling createComputePipeline(Async) validation for compute using <= devic
         entryPoint: 'main',
       },
     };
-    t.doCreateComputePipelineTest(isAsync, count <= countAtLimit, descriptor);
+    vtu.doCreateComputePipelineTest(t, isAsync, count <= countAtLimit, descriptor);
   });
 
 g.test('limits,invocations_per_workgroup')
@@ -214,7 +211,8 @@ Tests calling createComputePipeline(Async) validation for compute using <= devic
       },
     };
 
-    t.doCreateComputePipelineTest(
+    vtu.doCreateComputePipelineTest(
+      t,
       isAsync,
       size[0] * size[1] * size[2] <= t.device.limits.maxComputeInvocationsPerWorkgroup,
       descriptor
@@ -266,7 +264,7 @@ Tests calling createComputePipeline(Async) validation for compute workgroup_size
       workgroupX <= t.device.limits.maxComputeWorkgroupSizeX &&
       workgroupY <= t.device.limits.maxComputeWorkgroupSizeY &&
       workgroupZ <= t.device.limits.maxComputeWorkgroupSizeZ;
-    t.doCreateComputePipelineTest(isAsync, _success, descriptor);
+    vtu.doCreateComputePipelineTest(t, isAsync, _success, descriptor);
   });
 
 g.test('overrides,identifier')
@@ -321,7 +319,7 @@ Tests calling createComputePipeline(Async) validation for overridable constants
       },
     };
 
-    t.doCreateComputePipelineTest(isAsync, _success, descriptor);
+    vtu.doCreateComputePipelineTest(t, isAsync, _success, descriptor);
   });
 
 g.test('overrides,uninitialized')
@@ -379,7 +377,7 @@ Tests calling createComputePipeline(Async) validation for uninitialized overrida
       },
     };
 
-    t.doCreateComputePipelineTest(isAsync, _success, descriptor);
+    vtu.doCreateComputePipelineTest(t, isAsync, _success, descriptor);
   });
 
 g.test('overrides,value,type_error')
@@ -416,7 +414,7 @@ Tests calling createComputePipeline(Async) validation for constant values like i
       },
     };
 
-    t.doCreateComputePipelineTest(isAsync, _success, descriptor, 'TypeError');
+    vtu.doCreateComputePipelineTest(t, isAsync, _success, descriptor, 'TypeError');
   });
 
 g.test('overrides,value,validation_error')
@@ -477,7 +475,41 @@ TODO(#2060): test with last_castable_pipeline_override.
       },
     };
 
-    t.doCreateComputePipelineTest(isAsync, _success, descriptor);
+    vtu.doCreateComputePipelineTest(t, isAsync, _success, descriptor);
+  });
+
+g.test('overrides,entry_point,validation_error')
+  .desc(
+    `
+Tests that pipeline constant (override) errors only trigger on entry point usage.
+`
+  )
+  .params(u =>
+    u //
+      .combine('isAsync', [true, false])
+      .combine('pipeEntryPoint', ['main_success', 'main_pipe_error'])
+  )
+  .fn(t => {
+    const { isAsync, pipeEntryPoint } = t.params;
+    const descriptor = {
+      layout: 'auto' as const,
+      compute: {
+        module: t.device.createShaderModule({
+          code: `
+          override cu: u32 = 0u;
+          override cx: u32 = 1u/cu;
+          @compute @workgroup_size(1) fn main_success () {
+            _ = cu;
+          }
+          @compute @workgroup_size(1) fn main_pipe_error () {
+            _ = cx;
+          }`,
+        }),
+        entryPoint: pipeEntryPoint,
+      },
+    };
+
+    vtu.doCreateComputePipelineTest(t, isAsync, pipeEntryPoint === 'main_success', descriptor);
   });
 
 g.test('overrides,value,validation_error,f16')
@@ -515,10 +547,8 @@ clarity on whether values like f16.positive.last_castable_pipeline_override woul
         },
       ] as const)
   )
-  .beforeAllSubcases(t => {
-    t.selectDeviceOrSkipTestCase({ requiredFeatures: ['shader-f16'] });
-  })
   .fn(t => {
+    t.skipIfDeviceDoesNotHaveFeature('shader-f16');
     const { isAsync, constants, _success } = t.params;
 
     const descriptor = {
@@ -538,7 +568,7 @@ clarity on whether values like f16.positive.last_castable_pipeline_override woul
       },
     };
 
-    t.doCreateComputePipelineTest(isAsync, _success, descriptor);
+    vtu.doCreateComputePipelineTest(t, isAsync, _success, descriptor);
   });
 
 const kOverridesWorkgroupSizeShaders = {
@@ -592,7 +622,7 @@ Tests calling createComputePipeline(Async) validation for overridable constants
       },
     };
 
-    t.doCreateComputePipelineTest(isAsync, _success, descriptor);
+    vtu.doCreateComputePipelineTest(t, isAsync, _success, descriptor);
   });
 
 g.test('overrides,workgroup_size,limits')
@@ -627,7 +657,7 @@ Tests calling createComputePipeline(Async) validation for overridable constants
         },
       };
 
-      t.doCreateComputePipelineTest(isAsync, _success, descriptor);
+      vtu.doCreateComputePipelineTest(t, isAsync, _success, descriptor);
     };
 
     testFn(limits.maxComputeWorkgroupSizeX, 1, 1, true);
@@ -690,7 +720,7 @@ Tests calling createComputePipeline(Async) validation for overridable constants
         },
       };
 
-      t.doCreateComputePipelineTest(isAsync, _success, descriptor);
+      vtu.doCreateComputePipelineTest(t, isAsync, _success, descriptor);
     };
 
     testFn(1, 1, true);
@@ -718,6 +748,7 @@ g.test('resource_compatibility')
         !t.hasLanguageFeature('readonly_and_readwrite_storage_textures'),
       'Storage textures require language feature'
     );
+    t.skipIfTextureViewDimensionNotSupported(wgslResource.texture?.viewDimension);
 
     const layout = t.device.createPipelineLayout({
       bindGroupLayouts: [
@@ -734,7 +765,8 @@ g.test('resource_compatibility')
         entryPoint: 'main',
       },
     };
-    t.doCreateComputePipelineTest(
+    vtu.doCreateComputePipelineTest(
+      t,
       t.params.isAsync,
       doResourcesMatch(apiResource, wgslResource),
       descriptor
