@@ -4,9 +4,9 @@ Tests for validation in createQuerySet.
 
 import { makeTestGroup } from '../../../../common/framework/test_group.js';
 import { kQueryTypes, kMaxQueryCount } from '../../../capability_info.js';
-import { ValidationTest } from '../validation_test.js';
+import { AllFeaturesMaxLimitsGPUTest } from '../../../gpu_test.js';
 
-export const g = makeTestGroup(ValidationTest);
+export const g = makeTestGroup(AllFeaturesMaxLimitsGPUTest);
 
 g.test('count')
   .desc(
@@ -22,13 +22,11 @@ Tests that create query set with the count for all query types:
       .beginSubcases()
       .combine('count', [0, kMaxQueryCount, kMaxQueryCount + 1])
   )
-  .beforeAllSubcases(t => {
-    t.selectDeviceForQueryTypeOrSkipTestCase(t.params.type);
-  })
   .fn(t => {
     const { type, count } = t.params;
+    t.skipIfDeviceDoesNotSupportQueryType(type);
 
     t.expectValidationError(() => {
-      t.device.createQuerySet({ type, count });
+      t.createQuerySetTracked({ type, count });
     }, count > kMaxQueryCount);
   });
