@@ -1,7 +1,9 @@
+import { isCompatibilityDevice } from '../common/framework/test_config.js';
 import { keysOf } from '../common/util/data_tables.js';
 import { assert, unreachable } from '../common/util/util.js';
 
-import { align } from './util/math.js';
+import { align, roundDown } from './util/math.js';
+import { getTextureDimensionFromView } from './util/texture/base.js';
 import { ImageCopyType } from './util/texture/layout.js';
 
 //
@@ -23,8 +25,21 @@ const kFormatUniversalDefaults = {
   color: undefined,
   depth: undefined,
   stencil: undefined,
+  /**
+   * Info when this format can be used as a color render target. The format may require a feature
+   * to actually be used as a render target. Eg: rg11b10ufloat which requires rg11b10ufloat-renderable
+   * Call {@link isTextureFormatPossiblyUsableAsColorRenderAttachment} before having a device
+   * Call {@link isTextureFormatColorRenderable}(device, format) to find out for a particular device.
+   * Use {@link kPossibleColorRenderableTextureFormats} for params.
+   */
   colorRender: undefined,
-  /** Whether the format can be used in a multisample texture. */
+  /**
+   * Whether the format can possibly be used as a multisample texture. The format may require a
+   * feature to actually multisampled. Eg: rg11b10ufloat which requires rg11b10ufloat-renderable
+   * Call {@link isTextureFormatPossiblyMultisampled} before having a device
+   * Call {@link isTextureFormatMultisampled}(device, format) to find out for a particular device.
+   * Use {@link kPossibleMultisampledTextureFormats} for params.
+   */
   multisample: undefined,
   /** Optional feature required to use this format, or `undefined` if none. */
   feature: undefined,
@@ -552,7 +567,8 @@ const kRegularTextureFormatInfo = formatTableWithDefaults({
         readWriteStorage: false,
         bytes: 4,
       },
-      multisample: false,
+      colorRender: { blend: true, resolve: true, byteCost: 8, alignment: 4 },
+      multisample: true,
       /*prettier-ignore*/ get bytesPerBlock() { return this.color.bytes; },
     },
 
@@ -1391,7 +1407,7 @@ const kASTCTextureFormatInfo = formatTableWithDefaults({
   },
 } as const);
 
-// Definitions for use locally. To access the table entries, use `kTextureFormatInfo`.
+// Definitions for use locally.
 
 // MAINTENANCE_TODO: Consider generating the exports below programmatically by filtering the big list, instead
 // of using these local constants? Requires some type magic though.
@@ -1423,10 +1439,12 @@ const kASTCTextureFormatInfo = formatTableWithDefaults({
 /** An uncompressed (block size 1x1) format (regular | depth/stencil). */
 /* prettier-ignore */ export type UncompressedTextureFormat = keyof typeof kUncompressedTextureFormatInfo;
 
-/* prettier-ignore */ export const      kRegularTextureFormats: readonly      RegularTextureFormat[] = keysOf(     kRegularTextureFormatInfo);
-/* prettier-ignore */ export const   kSizedDepthStencilFormats: readonly   SizedDepthStencilFormat[] = keysOf(  kSizedDepthStencilFormatInfo);
-/* prettier-ignore */ export const kUnsizedDepthStencilFormats: readonly UnsizedDepthStencilFormat[] = keysOf(kUnsizedDepthStencilFormatInfo);
-/* prettier-ignore */ export const   kCompressedTextureFormats: readonly   CompressedTextureFormat[] = keysOf(  kCompressedTextureFormatInfo);
+/* prettier-ignore */ export const        kRegularTextureFormats: readonly      RegularTextureFormat[] = keysOf(     kRegularTextureFormatInfo);
+/* prettier-ignore */ export const     kSizedDepthStencilFormats: readonly   SizedDepthStencilFormat[] = keysOf(  kSizedDepthStencilFormatInfo);
+/* prettier-ignore */ export const   kUnsizedDepthStencilFormats: readonly UnsizedDepthStencilFormat[] = keysOf(kUnsizedDepthStencilFormatInfo);
+/* prettier-ignore */ export const     kCompressedTextureFormats: readonly   CompressedTextureFormat[] = keysOf(  kCompressedTextureFormatInfo);
+/* prettier-ignore */ export const   kBCCompressedTextureFormats: readonly   CompressedTextureFormat[] = keysOf(          kBCTextureFormatInfo);
+/* prettier-ignore */ export const kASTCCompressedTextureFormats: readonly   CompressedTextureFormat[] = keysOf(        kASTCTextureFormatInfo);
 
 /* prettier-ignore */ export const        kColorTextureFormats: readonly        ColorTextureFormat[] = keysOf(       kColorTextureFormatInfo);
 /* prettier-ignore */ export const    kEncodableTextureFormats: readonly    EncodableTextureFormat[] = keysOf(   kEncodableTextureFormatInfo);
@@ -1435,11 +1453,6 @@ const kASTCTextureFormatInfo = formatTableWithDefaults({
 /* prettier-ignore */ export const kUncompressedTextureFormats: readonly UncompressedTextureFormat[] = keysOf(kUncompressedTextureFormatInfo);
 /* prettier-ignore */ export const          kAllTextureFormats: readonly          GPUTextureFormat[] = keysOf(         kAllTextureFormatInfo);
 
-// CompressedTextureFormat are unrenderable so filter from RegularTextureFormats for color targets is enough
-export const kRenderableColorTextureFormats = kRegularTextureFormats.filter(
-  v => kColorTextureFormatInfo[v].colorRender
-);
-
 /** Per-GPUTextureFormat-per-aspect info. */
 interface TextureFormatAspectInfo {
   /** Whether the aspect can be used as `COPY_SRC`. */
@@ -1521,8 +1534,75 @@ type TextureFormatInfo_TypeCheck = {
     )
 );
 
-/** Per-GPUTextureFormat info. */
-export const kTextureFormatInfo = {
+/**
+ * DO NOT EXPORT THIS - functions that need info from this table should use the appropriate
+ * method for their needs.
+ *
+ * For a list of textures formats for test parameters there are:
+ *
+ * Lists of formats that might require features to be enabled
+ * * kPossibleColorRenderableTextureFormats
+ * * kPossibleStorageTextureFormats
+ * * kPossibleReadWriteStorageTextureFormats
+ * * kPossibleMultisampledTextureFormats
+ *
+ * Lists of formats that end in -srgb
+ * * kDifferentBaseFormatTextureFormats  (includes compressed textures)
+ * * kDifferentBaseFormatRegularTextureFormats (does not include compressed textures)
+ *
+ * Formats that require a feature to use at all (mostly compressed formats)
+ * * kOptionalTextureFormats
+ *
+ * Misc
+ * * kRegularTextureFormats
+ * * kSizedDepthStencilFormats
+ * * kUnsizedDepthStencilFormats
+ * * kCompressedTextureFormats
+ * * kUncompressedTextureFormats
+ * * kColorTextureFormats - color formats including compressed and sint/uint
+ * * kEncodableTextureFormats - formats that TexelView supports.
+ * * kSizedTextureFormats - formats that have a known size (so not depth24plus ...)
+ * * kDepthStencilFormats - depth, stencil, depth-stencil
+ * * kDepthTextureFormats - depth and depth-stencil
+ * * kStencilTextureFormats - stencil and depth-stencil
+ * * kAllTextureFormats
+ *
+ * If one of the list above does not work, add a new one or to filter in beforeAllSubcases you generally want to use
+ * You will not know if you can actually use a texture for the given use case until the test runs and has a device.
+ *
+ * * isTextureFormatPossiblyUsableAsRenderAttachment
+ * * isTextureFormatPossiblyUsableAsColorRenderAttachment
+ * * isTextureFormatPossiblyMultisampled
+ * * isTextureFormatPossiblyStorageReadable
+ * * isTextureFormatPossiblyStorageReadWritable
+ * * isTextureFormatPossiblyFilterableAsTextureF32
+ *
+ * These are also usable before or during a test
+ *
+ * * isColorTextureFormat
+ * * isDepthTextureFormat
+ * * isStencilTextureFormat
+ * * isDepthOrStencilTextureFormat
+ * * isEncodableTextureFormat
+ * * isRegularTextureFormat
+ * * isCompressedFloatTextureFormat
+ * * isSintOrUintFormat
+ *
+ * To skip a test use the `skipIfXXX` tests in `GPUTest` if possible. Otherwise these functions
+ * require a device to give a correct answer.
+ *
+ * * isTextureFormatUsableAsRenderAttachment
+ * * isTextureFormatColorRenderable
+ * * isTextureFormatResolvable
+ * * isTextureFormatBlendable
+ * * isTextureFormatMultisampled
+ * * isTextureFormatUsableAsStorageFormat
+ * * isTextureFormatUsableAsReadWriteStorageTexture
+ * * isTextureFormatUsableAsStorageFormatInCreateShaderModule
+ *
+ * Per-GPUTextureFormat info.
+ */
+const kTextureFormatInfo = {
   ...kRegularTextureFormatInfo,
   ...kSizedDepthStencilFormatInfo,
   ...kUnsizedDepthStencilFormatInfo,
@@ -1537,6 +1617,60 @@ const kTextureFormatInfo_TypeCheck: {
   readonly [F in GPUTextureFormat]: TextureFormatInfo_TypeCheck;
 } = kTextureFormatInfo;
 
+// Depth texture formats including formats that also support stencil
+export const kDepthTextureFormats = [
+  ...kDepthStencilFormats.filter(v => kTextureFormatInfo[v].depth),
+] as const;
+// Stencil texture formats including formats that also support depth
+export const kStencilTextureFormats = kDepthStencilFormats.filter(
+  v => kTextureFormatInfo[v].stencil
+);
+
+// Texture formats that may possibly be used as a storage texture.
+// Some may require certain features to be enabled.
+export const kPossibleStorageTextureFormats = [
+  ...kRegularTextureFormats.filter(f => kTextureFormatInfo[f].color?.storage),
+  'bgra8unorm',
+] as const;
+
+// Texture formats that may possibly be used as a storage texture.
+// Some may require certain features to be enabled.
+export const kPossibleReadWriteStorageTextureFormats = [
+  ...kPossibleStorageTextureFormats.filter(f => kTextureFormatInfo[f].color?.readWriteStorage),
+] as const;
+
+// Texture formats that may possibly be multisampled.
+// Some may require certain features to be enabled.
+export const kPossibleMultisampledTextureFormats = [
+  ...kRegularTextureFormats.filter(f => kTextureFormatInfo[f].multisample),
+  ...kDepthStencilFormats.filter(f => kTextureFormatInfo[f].multisample),
+] as const;
+
+// Texture formats that may possibly be color renderable.
+// Some may require certain features to be enabled.
+export const kPossibleColorRenderableTextureFormats = [
+  ...kRegularTextureFormats.filter(f => kTextureFormatInfo[f].colorRender),
+] as const;
+export type PossibleColorRenderTextureFormat =
+  (typeof kPossibleColorRenderableTextureFormats)[number];
+
+// Texture formats that have a different base format. This is effectively all -srgb formats
+// including compressed formats.
+export const kDifferentBaseFormatTextureFormats = kColorTextureFormats.filter(
+  f => kTextureFormatInfo[f].baseFormat && kTextureFormatInfo[f].baseFormat !== f
+);
+
+// "Regular" texture formats that have a different base format. This is effectively all -srgb formats
+// except compressed formats.
+export const kDifferentBaseFormatRegularTextureFormats = kRegularTextureFormats.filter(
+  f => kTextureFormatInfo[f].baseFormat && kTextureFormatInfo[f].baseFormat !== f
+);
+
+// Textures formats that are optional
+export const kOptionalTextureFormats = kAllTextureFormats.filter(
+  t => kTextureFormatInfo[t].feature !== undefined
+);
+
 /** Valid GPUTextureFormats for `copyExternalImageToTexture`, by spec. */
 export const kValidTextureFormatsForCopyE2T = [
   'r8unorm',
@@ -1732,10 +1866,13 @@ export function depthStencilFormatAspectSize(
  * Returns true iff a texture can be created with the provided GPUTextureDimension
  * (defaulting to 2d) and GPUTextureFormat, by spec.
  */
-export function textureDimensionAndFormatCompatible(
+export function textureFormatAndDimensionPossiblyCompatible(
   dimension: undefined | GPUTextureDimension,
   format: GPUTextureFormat
 ): boolean {
+  if (dimension === '3d' && (isBCTextureFormat(format) || isASTCTextureFormat(format))) {
+    return true;
+  }
   const info = kAllTextureFormatInfo[format];
   return !(
     (dimension === '1d' || dimension === '3d') &&
@@ -1744,16 +1881,164 @@ export function textureDimensionAndFormatCompatible(
 }
 
 /**
- * Check if two formats are view format compatible.
- *
- * This function may need to be generalized to use `baseFormat` from `kTextureFormatInfo`.
+ * Returns true iff a texture can be created with the provided GPUTextureDimension
+ * (defaulting to 2d) and GPUTextureFormat for a GPU device, by spec.
  */
-export function viewCompatible(
-  compatibilityMode: boolean,
+export function textureDimensionAndFormatCompatibleForDevice(
+  device: GPUDevice,
+  dimension: undefined | GPUTextureDimension,
+  format: GPUTextureFormat
+): boolean {
+  if (
+    dimension === '3d' &&
+    ((isBCTextureFormat(format) && device.features.has('texture-compression-bc-sliced-3d')) ||
+      (isASTCTextureFormat(format) && device.features.has('texture-compression-astc-sliced-3d')))
+  ) {
+    return true;
+  }
+  const info = kAllTextureFormatInfo[format];
+  return !(
+    (dimension === '1d' || dimension === '3d') &&
+    (info.blockWidth > 1 || info.depth || info.stencil)
+  );
+}
+
+/**
+ * Returns true iff a texture can be used with the provided GPUTextureViewDimension
+ */
+export function textureViewDimensionAndFormatCompatibleForDevice(
+  device: GPUDevice,
+  dimension: GPUTextureViewDimension,
+  format: GPUTextureFormat
+): boolean {
+  return textureDimensionAndFormatCompatibleForDevice(
+    device,
+    getTextureDimensionFromView(dimension),
+    format
+  );
+}
+
+/**
+ * Check if two formats are view format compatible.
+ */
+export function textureFormatsAreViewCompatible(
+  device: GPUDevice,
   a: GPUTextureFormat,
   b: GPUTextureFormat
-): boolean {
-  return compatibilityMode ? a === b : a === b || a + '-srgb' === b || b + '-srgb' === a;
+) {
+  return isCompatibilityDevice(device)
+    ? a === b
+    : a === b || a + '-srgb' === b || b + '-srgb' === a;
+}
+
+/**
+ * Gets the block width, height, and bytes per block for a color texture format.
+ * This is for color textures only. For all texture formats @see {@link getBlockInfoForTextureFormat}
+ * The point of this function is bytesPerBlock is always defined so no need to check that it's not
+ * vs getBlockInfoForTextureFormat where it may not be defined.
+ */
+export function getBlockInfoForColorTextureFormat(format: ColorTextureFormat) {
+  const info = kTextureFormatInfo[format];
+  return {
+    blockWidth: info.blockWidth,
+    blockHeight: info.blockHeight,
+    bytesPerBlock: info.color?.bytes,
+  };
+}
+
+/**
+ * Gets the block width, height, and bytes per block for a sized texture format.
+ * This is for sized textures only. For all texture formats @see {@link getBlockInfoForTextureFormat}
+ * The point of this function is bytesPerBlock is always defined so no need to check that it's not
+ * vs getBlockInfoForTextureFormat where it may not be defined.
+ */
+export function getBlockInfoForSizedTextureFormat(format: SizedTextureFormat) {
+  const info = kTextureFormatInfo[format];
+  const bytesPerBlock = info.color?.bytes || info.depth?.bytes || info.stencil?.bytes;
+  assert(!!bytesPerBlock);
+  return {
+    blockWidth: info.blockWidth,
+    blockHeight: info.blockHeight,
+    bytesPerBlock,
+  };
+}
+
+/**
+ * Gets the block width, height, and bytes per block for an encodable texture format.
+ * This is for encodable textures only. For all texture formats @see {@link getBlockInfoForTextureFormat}
+ * The point of this function is bytesPerBlock is always defined so no need to check that it's not
+ * vs getBlockInfoForTextureFormat where it may not be defined.
+ */
+export function getBlockInfoForEncodableTextureFormat(format: EncodableTextureFormat) {
+  const info = kTextureFormatInfo[format];
+  const bytesPerBlock = info.color?.bytes || info.depth?.bytes || info.stencil?.bytes;
+  assert(!!bytesPerBlock);
+  return {
+    blockWidth: info.blockWidth,
+    blockHeight: info.blockHeight,
+    bytesPerBlock,
+  };
+}
+
+/**
+ * Gets the block width, height, and bytes per block for a color texture format.
+ * Note that bytesPerBlock will be undefined if format's size is undefined.
+ * If you are only using color or encodable formats, @see {@link getBlockInfoForColorTextureFormat}
+ * or {@link getBlockInfoForEncodableTextureFormat}
+ */
+export function getBlockInfoForTextureFormat(format: GPUTextureFormat) {
+  const info = kTextureFormatInfo[format];
+  return {
+    blockWidth: info.blockWidth,
+    blockHeight: info.blockHeight,
+    bytesPerBlock: info.color?.bytes ?? info.depth?.bytes ?? info.stencil?.bytes,
+  };
+}
+
+/**
+ * Returns the "byteCost" of rendering to a color texture format.
+ */
+export function getColorRenderByteCost(format: PossibleColorRenderTextureFormat) {
+  const byteCost = kTextureFormatInfo[format].colorRender?.byteCost;
+  // MAINTENANCE_TODO: remove this assert. The issue is typescript thinks
+  // PossibleColorRenderTextureFormat contains all texture formats and not just
+  // a filtered list.
+  assert(byteCost !== undefined);
+  return byteCost;
+}
+
+/**
+ * Returns the "alignment" of rendering to a color texture format.
+ */
+export function getColorRenderAlignment(format: PossibleColorRenderTextureFormat) {
+  const alignment = kTextureFormatInfo[format].colorRender?.alignment;
+  // MAINTENANCE_TODO: remove this assert. The issue is typescript thinks
+  // PossibleColorRenderTextureFormat contains all texture formats and not just
+  // a filtered list.
+  assert(alignment !== undefined);
+  return alignment;
+}
+
+/**
+ * Gets the baseFormat for a texture format.
+ */
+export function getBaseFormatForTextureFormat(
+  format: (typeof kDifferentBaseFormatTextureFormats)[number]
+): ColorTextureFormat {
+  return kTextureFormatInfo[format].baseFormat!;
+}
+
+export function getBaseFormatForRegularTextureFormat(
+  format: RegularTextureFormat
+): RegularTextureFormat | undefined {
+  return kTextureFormatInfo[format].baseFormat as RegularTextureFormat;
+}
+
+/**
+ * Gets the feature needed for a give texture format or undefined if none.
+ */
+export function getRequiredFeatureForTextureFormat(format: GPUTextureFormat) {
+  return kTextureFormatInfo[format].feature;
 }
 
 export function getFeaturesForFormats<T>(
@@ -1769,32 +2054,362 @@ export function filterFormatsByFeature<T>(
   return formats.filter(f => f === undefined || kTextureFormatInfo[f].feature === feature);
 }
 
+export function canCopyToAspectOfTextureFormat(format: GPUTextureFormat, aspect: GPUTextureAspect) {
+  const info = kTextureFormatInfo[format];
+  switch (aspect) {
+    case 'depth-only':
+      assert(isDepthTextureFormat(format));
+      return info.depth && info.depth.copyDst;
+    case 'stencil-only':
+      assert(isStencilTextureFormat(format));
+      return info.stencil && info.stencil.copyDst;
+    case 'all':
+      return (
+        (!isDepthTextureFormat(format) || info.depth?.copyDst) &&
+        (!isStencilTextureFormat(format) || info.stencil?.copyDst) &&
+        (!isColorTextureFormat(format) || !info.color?.copyDst)
+      );
+  }
+}
+
+export function canCopyFromAspectOfTextureFormat(
+  format: GPUTextureFormat,
+  aspect: GPUTextureAspect
+) {
+  const info = kTextureFormatInfo[format];
+  switch (aspect) {
+    case 'depth-only':
+      assert(isDepthTextureFormat(format));
+      return info.depth && info.depth.copySrc;
+    case 'stencil-only':
+      assert(isStencilTextureFormat(format));
+      return info.stencil && info.stencil.copySrc;
+    case 'all':
+      return (
+        (!isDepthTextureFormat(format) || info.depth?.copySrc) &&
+        (!isStencilTextureFormat(format) || info.stencil?.copySrc) &&
+        (!isColorTextureFormat(format) || !info.color?.copySrc)
+      );
+  }
+}
+
+/**
+ * Returns true if all aspects of texture can be copied to (used with COPY_DST)
+ */
+export function canCopyToAllAspectsOfTextureFormat(format: GPUTextureFormat) {
+  const info = kTextureFormatInfo[format];
+  return (
+    (!info.color || info.color.copyDst) &&
+    (!info.depth || info.depth.copyDst) &&
+    (!info.stencil || info.stencil.copyDst)
+  );
+}
+
+/**
+ * Returns true if all aspects of texture can be copied from (used with COPY_SRC)
+ */
+export function canCopyFromAllAspectsOfTextureFormat(format: GPUTextureFormat) {
+  const info = kTextureFormatInfo[format];
+  return (
+    (!info.color || info.color.copySrc) &&
+    (!info.depth || info.depth.copySrc) &&
+    (!info.stencil || info.stencil.copySrc)
+  );
+}
+
 export function isCompressedTextureFormat(format: GPUTextureFormat) {
   return format in kCompressedTextureFormatInfo;
 }
 
+export function isBCTextureFormat(format: GPUTextureFormat) {
+  return format in kBCTextureFormatInfo;
+}
+
+export function isASTCTextureFormat(format: GPUTextureFormat) {
+  return format in kASTCTextureFormatInfo;
+}
+
+export function isColorTextureFormat(format: GPUTextureFormat) {
+  return !!kTextureFormatInfo[format].color;
+}
+
+export function isDepthTextureFormat(format: GPUTextureFormat) {
+  return !!kTextureFormatInfo[format].depth;
+}
+
+export function isStencilTextureFormat(format: GPUTextureFormat) {
+  return !!kTextureFormatInfo[format].stencil;
+}
+
+export function isDepthOrStencilTextureFormat(format: GPUTextureFormat) {
+  return isDepthTextureFormat(format) || isStencilTextureFormat(format);
+}
+
+export function isEncodableTextureFormat(format: GPUTextureFormat) {
+  return kEncodableTextureFormats.includes(format as EncodableTextureFormat);
+}
+
+/**
+ * Returns if a texture can be used as a render attachment. some color formats and all
+ * depth textures and stencil textures are usable with usage RENDER_ATTACHMENT.
+ */
+export function isTextureFormatUsableAsRenderAttachment(
+  device: GPUDevice,
+  format: GPUTextureFormat
+) {
+  if (format === 'rg11b10ufloat') {
+    return device.features.has('rg11b10ufloat-renderable');
+  }
+  return kTextureFormatInfo[format].colorRender || isDepthOrStencilTextureFormat(format);
+}
+
+/**
+ * Returns if a texture can be used as a "colorAttachment".
+ */
+export function isTextureFormatColorRenderable(
+  device: GPUDevice,
+  format: GPUTextureFormat
+): boolean {
+  if (format === 'rg11b10ufloat') {
+    return device.features.has('rg11b10ufloat-renderable');
+  }
+  return !!kAllTextureFormatInfo[format].colorRender;
+}
+
+/**
+ * Returns if a texture can be blended.
+ */
+export function isTextureFormatBlendable(device: GPUDevice, format: GPUTextureFormat): boolean {
+  if (!isTextureFormatColorRenderable(device, format)) {
+    return false;
+  }
+  if (format === 'rg11b10ufloat') {
+    return device.features.has('rg11b10ufloat-renderable');
+  }
+  if (is32Float(format)) {
+    return device.features.has('float32-blendable');
+  }
+  return !!kAllTextureFormatInfo[format].colorRender?.blend;
+}
+
+/**
+ * Returns the texture's type (float, unsigned-float, sint, uint, depth)
+ */
+export function getTextureFormatType(format: GPUTextureFormat) {
+  const info = kTextureFormatInfo[format];
+  const type = info.color?.type ?? info.depth?.type ?? info.stencil?.type;
+  assert(!!type);
+  return type;
+}
+
+/**
+ * Returns the regular texture's type (float, unsigned-float, sint, uint)
+ */
+export function getTextureFormatColorType(format: RegularTextureFormat) {
+  const info = kTextureFormatInfo[format];
+  const type = info.color?.type;
+  assert(!!type);
+  return type;
+}
+
+/**
+ * Returns true if a texture can possibly be used as a render attachment.
+ * The texture may require certain features to be enabled.
+ */
+export function isTextureFormatPossiblyUsableAsRenderAttachment(format: GPUTextureFormat) {
+  const info = kTextureFormatInfo[format];
+  return isDepthOrStencilTextureFormat(format) || !!info.colorRender;
+}
+
+/**
+ * Returns true if a texture can possibly be used as a color render attachment.
+ * The texture may require certain features to be enabled.
+ */
+export function isTextureFormatPossiblyUsableAsColorRenderAttachment(format: GPUTextureFormat) {
+  const info = kTextureFormatInfo[format];
+  return !!info.colorRender;
+}
+
+/**
+ * Returns true if a texture can possibly be used multisampled.
+ * The texture may require certain features to be enabled.
+ */
+export function isTextureFormatPossiblyMultisampled(format: GPUTextureFormat) {
+  const info = kTextureFormatInfo[format];
+  return info.multisample;
+}
+
+/**
+ * Returns true if a texture can possibly be used as a storage texture.
+ * The texture may require certain features to be enabled.
+ */
+export function isTextureFormatPossiblyStorageReadable(format: GPUTextureFormat) {
+  return !!kTextureFormatInfo[format].color?.storage;
+}
+
+/**
+ * Returns true if a texture can possibly be used as a read-write storage texture.
+ * The texture may require certain features to be enabled.
+ */
+export function isTextureFormatPossiblyStorageReadWritable(format: GPUTextureFormat) {
+  return !!kTextureFormatInfo[format].color?.readWriteStorage;
+}
+
+export function is16Float(format: GPUTextureFormat) {
+  return format === 'r16float' || format === 'rg16float' || format === 'rgba16float';
+}
+
+export function is32Float(format: GPUTextureFormat) {
+  return format === 'r32float' || format === 'rg32float' || format === 'rgba32float';
+}
+
+/**
+ * Returns true if texture is filterable as `texture_xxx<f32>`
+ *
+ * examples:
+ * * 'rgba8unorm' -> true
+ * * 'depth16unorm' -> false
+ * * 'rgba32float' -> true (you need to enable feature 'float32-filterable')
+ */
+export function isTextureFormatPossiblyFilterableAsTextureF32(format: GPUTextureFormat) {
+  const info = kTextureFormatInfo[format];
+  return info.color?.type === 'float' || is32Float(format);
+}
+
 export const kCompatModeUnsupportedStorageTextureFormats: readonly GPUTextureFormat[] = [
   'rg32float',
   'rg32sint',
   'rg32uint',
 ] as const;
 
+/**
+ * Return true if the format can be used as a storage texture.
+ * Note: Some formats can be compiled in a shader but can not be used
+ * in a pipeline or elsewhere. This function returns whether or not the format
+ * can be used in general. If you want to know if the format can used when compiling
+ * a shader @see {@link isTextureFormatUsableAsStorageFormatInCreateShaderModule}
+ */
 export function isTextureFormatUsableAsStorageFormat(
-  format: GPUTextureFormat,
-  isCompatibilityMode: boolean
-) {
-  if (isCompatibilityMode) {
+  device: GPUDevice,
+  format: GPUTextureFormat
+): boolean {
+  if (isCompatibilityDevice(device)) {
     if (kCompatModeUnsupportedStorageTextureFormats.indexOf(format) >= 0) {
       return false;
     }
   }
-  return !!kTextureFormatInfo[format].color?.storage;
+  if (format === 'bgra8unorm' && device.features.has('bgra8unorm-storage')) {
+    return true;
+  }
+  const info = kTextureFormatInfo[format];
+  return !!(info.color?.storage || info.depth?.storage || info.stencil?.storage);
+}
+
+/**
+ * Returns true if format can be used with createShaderModule on the device.
+ * Some formats may require a feature to be enabled before they can be used
+ * as a storage texture. Others, can't be used in a pipeline but can be compiled
+ * in a shader. Examples are rg32float, rg32uint, rg32sint which are not usable
+ * in compat mode but shaders can be compiled. Similarly, bgra8unorm can be
+ * compiled but can't be used in a pipeline unless feature 'bgra8unorm-storage'
+ * is available.
+ */
+export function isTextureFormatUsableAsStorageFormatInCreateShaderModule(
+  device: GPUDevice,
+  format: GPUTextureFormat
+): boolean {
+  if (format === 'bgra8unorm') {
+    return true;
+  }
+  const info = kTextureFormatInfo[format];
+  return !!(info.color?.storage || info.depth?.storage || info.stencil?.storage);
+}
+
+export function isTextureFormatUsableAsReadWriteStorageTexture(
+  device: GPUDevice,
+  format: GPUTextureFormat
+): boolean {
+  return (
+    isTextureFormatUsableAsStorageFormat(device, format) &&
+    !!kTextureFormatInfo[format].color?.readWriteStorage
+  );
 }
 
 export function isRegularTextureFormat(format: GPUTextureFormat) {
   return format in kRegularTextureFormatInfo;
 }
 
+/**
+ * Returns true if format is both compressed and a float format, for example 'bc6h-rgb-ufloat'.
+ */
+export function isCompressedFloatTextureFormat(format: GPUTextureFormat) {
+  return isCompressedTextureFormat(format) && format.includes('float');
+}
+
+/**
+ * Returns true if format is sint or uint
+ */
+export function isSintOrUintFormat(format: GPUTextureFormat) {
+  const info = kTextureFormatInfo[format];
+  const type = info.color?.type ?? info.depth?.type ?? info.stencil?.type;
+  return type === 'sint' || type === 'uint';
+}
+
+/**
+ * Returns true if format can be multisampled.
+ */
+export const kCompatModeUnsupportedMultisampledTextureFormats: readonly GPUTextureFormat[] = [
+  'r8uint',
+  'r8sint',
+  'rg8uint',
+  'rg8sint',
+  'rgba8uint',
+  'rgba8sint',
+  'r16uint',
+  'r16sint',
+  'rg16uint',
+  'rg16sint',
+  'rgba16uint',
+  'rgba16sint',
+  'rgb10a2uint',
+  'rgba16float',
+  'r32float',
+] as const;
+
+/**
+ * Returns true if you can make a multisampled texture from the given format.
+ */
+export function isTextureFormatMultisampled(device: GPUDevice, format: GPUTextureFormat): boolean {
+  if (isCompatibilityDevice(device)) {
+    if (kCompatModeUnsupportedMultisampledTextureFormats.indexOf(format) >= 0) {
+      return false;
+    }
+  }
+  if (format === 'rg11b10ufloat') {
+    return device.features.has('rg11b10ufloat-renderable');
+  }
+  return kAllTextureFormatInfo[format].multisample;
+}
+
+/**
+ * Returns true if a texture can be "resolved". uint/sint formats can be multisampled but
+ * can not be resolved.
+ */
+export function isTextureFormatResolvable(device: GPUDevice, format: GPUTextureFormat): boolean {
+  if (format === 'rg11b10ufloat') {
+    return device.features.has('rg11b10ufloat-renderable');
+  }
+  // You can't resolve a non-multisampled format.
+  if (!isTextureFormatMultisampled(device, format)) {
+    return false;
+  }
+  const info = kAllTextureFormatInfo[format];
+  return !!info.colorRender?.resolve;
+}
+
+// MAINTENANCE_TODD: See if we can remove this. This doesn't seem useful since
+// formats are not on/off by feature. Some are on but a feature allows them to be
+// used in more cases, like going from un-renderable to renderable, etc...
 export const kFeaturesForFormats = getFeaturesForFormats(kAllTextureFormats);
 
 /**
@@ -1803,6 +2418,10 @@ export const kFeaturesForFormats = getFeaturesForFormats(kAllTextureFormats);
 export function computeBytesPerSampleFromFormats(formats: readonly GPUTextureFormat[]) {
   let bytesPerSample = 0;
   for (const format of formats) {
+    // MAINTENANCE_TODO: Add colorRender to rg11b10ufloat format in kTextureFormatInfo
+    // The issue is if we add it now lots of tests will break as they'll think they can
+    // render to the format but are not enabling 'rg11b10ufloat-renderable'. Once we
+    // get the CTS refactored (see issue 4181), then fix this.
     const info = kTextureFormatInfo[format];
     const alignedBytesPerSample = align(bytesPerSample, info.colorRender!.alignment);
     bytesPerSample = alignedBytesPerSample + info.colorRender!.byteCost;
@@ -1816,3 +2435,33 @@ export function computeBytesPerSampleFromFormats(formats: readonly GPUTextureFor
 export function computeBytesPerSample(targets: GPUColorTargetState[]) {
   return computeBytesPerSampleFromFormats(targets.map(({ format }) => format));
 }
+
+/**
+ * Returns the maximum valid size in each dimension for a given texture format.
+ * This is useful because compressed formats must be a multiple of blocks in size
+ * so, for example, the largest valid width of a 2d texture
+ * roundDown(device.limits.maxTextureDimension2D, blockWidth)
+ */
+export function getMaxValidTextureSizeForFormatAndDimension(
+  device: GPUDevice,
+  format: GPUTextureFormat,
+  dimension: GPUTextureDimension
+): [number, number, number] {
+  const info = getBlockInfoForTextureFormat(format);
+  switch (dimension) {
+    case '1d':
+      return [device.limits.maxTextureDimension1D, 1, 1];
+    case '2d':
+      return [
+        roundDown(device.limits.maxTextureDimension2D, info.blockWidth),
+        roundDown(device.limits.maxTextureDimension2D, info.blockHeight),
+        device.limits.maxTextureArrayLayers,
+      ];
+    case '3d':
+      return [
+        roundDown(device.limits.maxTextureDimension3D, info.blockWidth),
+        roundDown(device.limits.maxTextureDimension3D, info.blockHeight),
+        device.limits.maxTextureDimension3D,
+      ];
+  }
+}
