@@ -16,7 +16,7 @@ component of the result vector equals cmp.
 import { makeTestGroup } from '../../../../../../../common/framework/test_group.js';
 import { keysOf } from '../../../../../../../common/util/data_tables.js';
 import { assert } from '../../../../../../../common/util/util.js';
-import { GPUTest } from '../../../../../../gpu_test.js';
+import { AllFeaturesMaxLimitsGPUTest } from '../../../../../../gpu_test.js';
 
 import {
   dispatchSizes,
@@ -26,7 +26,7 @@ import {
   onlyWorkgroupSizes,
 } from './harness.js';
 
-export const g = makeTestGroup(GPUTest);
+export const g = makeTestGroup(AllFeaturesMaxLimitsGPUTest);
 
 g.test('compare_exchange_weak_storage_basic')
   .specURL('https://www.w3.org/TR/WGSL/#atomic-rmw')
@@ -102,27 +102,24 @@ struct __atomic_compare_exchange_result<T> {
     const arrayType = typedArrayCtor(scalarType);
 
     // Create input buffer with values [0..n]
-    const inputBuffer = t.device.createBuffer({
+    const inputBuffer = t.createBufferTracked({
       size: bufferNumElements * arrayType.BYTES_PER_ELEMENT,
       usage: GPUBufferUsage.STORAGE | GPUBufferUsage.COPY_SRC,
       mappedAtCreation: true,
     });
-    t.trackForCleanup(inputBuffer);
     const data = new arrayType(inputBuffer.getMappedRange());
     data.forEach((_, i) => (data[i] = i));
     inputBuffer.unmap();
 
-    const outputBuffer = t.device.createBuffer({
+    const outputBuffer = t.createBufferTracked({
       size: bufferNumElements * arrayType.BYTES_PER_ELEMENT,
       usage: GPUBufferUsage.STORAGE | GPUBufferUsage.COPY_SRC,
     });
-    t.trackForCleanup(outputBuffer);
 
-    const exchangedBuffer = t.device.createBuffer({
+    const exchangedBuffer = t.createBufferTracked({
       size: bufferNumElements * arrayType.BYTES_PER_ELEMENT,
       usage: GPUBufferUsage.STORAGE | GPUBufferUsage.COPY_SRC,
     });
-    t.trackForCleanup(exchangedBuffer);
 
     const bindGroup = t.device.createBindGroup({
       layout: pipeline.getBindGroupLayout(0),
@@ -252,23 +249,20 @@ struct __atomic_compare_exchange_result<T> {
 
     const arrayType = typedArrayCtor(scalarType);
 
-    const outputBuffer = t.device.createBuffer({
+    const outputBuffer = t.createBufferTracked({
       size: wgNumElements * dispatchSize * arrayType.BYTES_PER_ELEMENT,
       usage: GPUBufferUsage.STORAGE | GPUBufferUsage.COPY_SRC,
     });
-    t.trackForCleanup(outputBuffer);
 
-    const wgCopyBuffer = t.device.createBuffer({
+    const wgCopyBuffer = t.createBufferTracked({
       size: wgNumElements * dispatchSize * arrayType.BYTES_PER_ELEMENT,
       usage: GPUBufferUsage.STORAGE | GPUBufferUsage.COPY_SRC,
     });
-    t.trackForCleanup(outputBuffer);
 
-    const exchangedBuffer = t.device.createBuffer({
+    const exchangedBuffer = t.createBufferTracked({
       size: wgNumElements * dispatchSize * arrayType.BYTES_PER_ELEMENT,
       usage: GPUBufferUsage.STORAGE | GPUBufferUsage.COPY_SRC,
     });
-    t.trackForCleanup(exchangedBuffer);
 
     const bindGroup = t.device.createBindGroup({
       layout: pipeline.getBindGroupLayout(0),
@@ -414,7 +408,7 @@ struct __atomic_compare_exchange_result<T> {
     const defaultValue = 99999999;
 
     // Create single-value data buffer initialized to the first ping-pong value
-    const dataBuffer = t.device.createBuffer({
+    const dataBuffer = t.createBufferTracked({
       size: 1 * arrayType.BYTES_PER_ELEMENT,
       usage: GPUBufferUsage.STORAGE | GPUBufferUsage.COPY_SRC,
       mappedAtCreation: true,
@@ -424,26 +418,23 @@ struct __atomic_compare_exchange_result<T> {
       data[0] = pingPongValues[0];
       dataBuffer.unmap();
     }
-    t.trackForCleanup(dataBuffer);
 
-    const oldValuesBuffer = t.device.createBuffer({
+    const oldValuesBuffer = t.createBufferTracked({
       size: bufferNumElements * arrayType.BYTES_PER_ELEMENT,
       usage: GPUBufferUsage.STORAGE | GPUBufferUsage.COPY_SRC,
       mappedAtCreation: true,
     });
-    t.trackForCleanup(oldValuesBuffer);
     {
       const data = new arrayType(oldValuesBuffer.getMappedRange());
       data.fill(defaultValue);
       oldValuesBuffer.unmap();
     }
 
-    const exchangedBuffer = t.device.createBuffer({
+    const exchangedBuffer = t.createBufferTracked({
       size: bufferNumElements * arrayType.BYTES_PER_ELEMENT,
       usage: GPUBufferUsage.STORAGE | GPUBufferUsage.COPY_SRC,
       mappedAtCreation: true,
     });
-    t.trackForCleanup(exchangedBuffer);
     {
       const data = new arrayType(exchangedBuffer.getMappedRange());
       data.fill(defaultValue);
@@ -641,24 +632,22 @@ struct __atomic_compare_exchange_result<T> {
     const arrayType = typedArrayCtor(scalarType);
     const defaultValue = 99999999;
 
-    const oldValuesBuffer = t.device.createBuffer({
+    const oldValuesBuffer = t.createBufferTracked({
       size: bufferNumElements * arrayType.BYTES_PER_ELEMENT,
       usage: GPUBufferUsage.STORAGE | GPUBufferUsage.COPY_SRC,
       mappedAtCreation: true,
     });
-    t.trackForCleanup(oldValuesBuffer);
     {
       const data = new arrayType(oldValuesBuffer.getMappedRange());
       data.fill(defaultValue);
       oldValuesBuffer.unmap();
     }
 
-    const exchangedBuffer = t.device.createBuffer({
+    const exchangedBuffer = t.createBufferTracked({
       size: bufferNumElements * arrayType.BYTES_PER_ELEMENT,
       usage: GPUBufferUsage.STORAGE | GPUBufferUsage.COPY_SRC,
       mappedAtCreation: true,
     });
-    t.trackForCleanup(exchangedBuffer);
     {
       const data = new arrayType(exchangedBuffer.getMappedRange());
       data.fill(defaultValue);
